@@ -57,6 +57,7 @@ def atrium_sud(username, password):
     response = session.post(ent_login, headers=headers, data=payload, cookies=cookies)
     return requests.utils.cookiejar_from_dict(requests.utils.dict_from_cookiejar(session.cookies))
 
+
 def ac_reims(username, password):
     """
     ENT for AC Reims
@@ -111,6 +112,7 @@ def ac_reims(username, password):
     cookies = requests.utils.cookiejar_from_dict(requests.utils.dict_from_cookiejar(session.cookies))
     return cookies
 
+
 def occitanie_montpellier(username, password):
     """
     ENT for Occitanie Montpellier
@@ -164,3 +166,61 @@ def occitanie_montpellier(username, password):
     cookies = requests.utils.cookiejar_from_dict(requests.utils.dict_from_cookiejar(session.cookies))
     return cookies
 
+
+def ac_reunion(username, password):
+    """
+    ENT for AC Reunion
+
+    Parameters
+    ----------
+    username : str
+        username
+    password : str
+        password
+
+    Returns
+    -------
+    cookies : cookies
+        returns the ent session cookies
+    """
+    # ENT / PRONOTE required URLs
+    ent_login = 'https://portail.college-jeandesme.re:8443/login?service=https:%2F%2Fportail.college-jeandesme.re%2Fpronote%2Feleve.html'
+
+    # Required Headers
+    headers = {
+        'connection': 'keep-alive',
+        'User-Agent': 'Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:73.0) Gecko/20100101 Firefox/73.0'}
+
+    # ENT Connection
+    session = requests.Session()
+    response = session.get(ent_login, headers=headers)
+
+    log.debug('[ENT Reunion] Logging in with ' + username)
+
+    # Login payload
+    soup = BeautifulSoup(response.text, 'html.parser')
+    input_ = soup.find('input', {'type': 'hidden', 'name': 'lt'})
+    lt = input_.get('value')
+
+    payload = {
+        'service': 'https://portail.college-jeandesme.re/pronote/eleve.html',
+        'lt': lt,
+        'previous_user': username + '@default',
+        'username': username,
+        'password': password}
+
+    # Send user:pass to the ENT
+    cookies = requests.utils.cookiejar_from_dict(requests.utils.dict_from_cookiejar(session.cookies))
+    response = session.post(ent_login, headers=headers, data=payload, cookies=cookies)
+
+    new_url = response.url
+
+    cookies = requests.utils.cookiejar_from_dict(requests.utils.dict_from_cookiejar(session.cookies))
+    response = session.get(new_url, headers=headers, cookies=cookies)
+
+    pronote_url = response.url
+
+    cookies = requests.utils.cookiejar_from_dict(requests.utils.dict_from_cookiejar(session.cookies))
+    response = session.get(pronote_url, headers=headers, cookies=cookies)
+
+    return requests.utils.cookiejar_from_dict(requests.utils.dict_from_cookiejar(session.cookies))
\ No newline at end of file
